# Create-Sequence.ps1
# Cre un objet squence similaire  une squence Oracle

# L'appel de Nextval n'est pas ncessaire CurrVal est accessible ds que l'objet cr
   

#Proprits en ReadOnly: 
#-----------------------------------------------------------------------------------------
#Doc US : http://www.acs.ilstu.edu/docs/oracle/server.101/b10759/statements_6014.htm
# Name         : Nom de la squence.
#
# CurrVal      : Contient la valeur courante. 
#
# Increment_By : Spcifie l'interval entre les numros de la squence. 
#                Cette valeur entire peut tre n'importe quel nombre entier positif ou ngatif de type .NET INT, mais elle ne peut pas tre 0. 
#                L'absolu de cette valeur doit tre moins (ou gal) que la diffrence de MAXVALUE et de MINVALUE. 
#                Si cette valeur est ngative, alors le squence est descendante (ordre dcroissant). 
#                Si la valeur est positive, alors la squence est ascendante (ordres croissant). 
#                Si vous omettez ce paramtre la valeur de l'interval est par dfaut de 1.
#
# Start_With   : Spcifie le premier nombre de la squence  produire. 
#                Employez ce paramtre pour dmarrer une squence ascendante  une valeur plus grande que son minimum ou pour  
#                dmarrer une squence descendante  une valeur moindre que son maximum. Pour des squences ascendantes, 
#                la valeur par dfaut est la valeur minimum de la squence. Pour des squences descendantes, la valeur par dfaut 
#                est la valeur maximum de la squence.
#                Note :
#                 Cette valeur n'est pas ncessairement la valeur  laquelle une squence ascendante cyclique redmarre une fois 
#                 sa valeur maximum ou minimum atteinte. 
#
# MaxValue     : Spcifie la valeur maximum que la squence peut produire. 
#                MAXVALUE doit tre gal ou plus grand que le valeur du paramtre START_WITH et doit tre plus grand que MINVALUE.
#
# MinValue     : Spcifie la valeur minimum de la squence. 
#                MINVALUE doit tre infrieur ou gal  le valeur du paramtre START_WITH  et doit tre infrieure  MAXVALUE.
#
# Cycle        : Indique que la squence continue  produire des valeurs une fois atteinte sa valeur maximum ou minimum. 
#                Une fois qu'une squence ascendante a atteint sa valeur maximum, elle reprend  sa valeur minimum. 
#                Une fois qu'une squence descendante a atteint sa valeur minimum, elle reprend  sa valeur maximum.
#                Par dfaut une squence ne produit plus de valeurs une fois atteinte sa valeur maximum ou minimum. 
#
# Comment      : Commentaire.
#
#Mthodes :
#-----------------------------------------------------------------------------------------
# NextVal: Incrmente la squence et retourne la nouvelle valeur.



# ** Le fonctionnment de cet objet squence est similaire  celui d'une squence Oracle sans pour autant tre identique.

# Si vous ne spcifiez aucun paramtre, autre que le nom obligatoire, alors vous crez une squence ascendante qui 
# dbute  1 et est incrmente de 1 jusqu' sa limite suprieure ([int]::MaxValue). 
# Si vous spcifiez seulement INCREMENT_BY -1 vous crez une squence dscendante qui dbute  -1 et 
# est dcrmente de 1 jusqu' sa limite infrieure ([int]::MinValue).

# Pour crer une squence ascendante qui incrmente sans limite (autre que celle du type .NET INT), omettez le paramtre MAXVALUE. 
# Pour crer une squence descendante qui dcrmente sans limite, omettez le paramtre MINVALUE.

# Pour crer une squence ascendante qui s'arrte  une limite prdfinie, spcifiez une valeur pour le paramtre MAXVALUE. 
# Pour crer une squence descendante qui s'arrte  une limite prdfinie, spcifiez une valeur pour le paramtre MINVALUE. 
# Si vous ne prcisez pas le paramtre -CYCLE, n'importe quelle tentative de produire un numro de squence une fois que la squence 
# a atteint sa limite dclenchera une erreur.

# Pour crer une squence qui redmarre/boucle aprs avoir atteint une limite prdfinie, indiquez le paramtre CYCLE. Dans ce cas 
# vous devez obligatoirement spcifiez une valeur pour les paramtres MAXVALUE ou MINVALUE. 

#Valeur par dfaut d'une squence ascendante :
# $Sq=Create-Sequence "SEQ_Test"  ;$Sq
#
# Name         : SEQ_Test
# CurrVal      : 1
# Increment_By : 1
# MaxValue     : 2147483647
# MinValue     : 1
# Start_With   : 1
# Cycle        : False
# Comment      :

#Valeur par dfaut d'une squence descendante :
# $Sq=Create-Sequence "SEQ_Test" -inc -1  ;$Sq
# 
# Name         : SEQ_Test
# CurrVal      : -1
# Increment_By : -1
# MaxValue     : -1
# MinValue     : -2147483648
# Start_With   : -1
# Cycle        : False
# Comment      :

#Exemple :
# $DebugPreference = "Continue"
# $Sq= Create-Sequence "SEQ_Test"
# $Sq.Currval
# $Sq.NextVal()
# $Sq.Currval

Function Create-Sequence
{
  #Tous les paramtres sont contraint, i.e. on prcise un type. 
 param([String] $Sequence_Name, 
       [String] $Comment,
       [int] $Increment_By=1,
       [int] $MaxValue,
       [int] $MinValue,
       [int] $Start_With,            
       [switch] $Cycle)
  
  write-Debug "[Valeurs des paramtres]"
  write-Debug "Sequence_Name $Sequence_Name"
  write-Debug "Comment $Comment"
  write-Debug "Increment_By $Increment_By" 
  write-Debug "MaxValue $MaxValue"
  write-Debug "MinValue $MinValue"
  write-Debug "Start_With $Start_With"  
  write-Debug "Cycle $Cycle"        

  if (($MyInvocation.UnBoundArguments).count -ne 0)
   {Throw "Le ou les paramtres suivants sont inconnus : $($MyInvocation.UnBoundArguments)."}
  
 if (($Sequence_Name -eq $null) -or ($Sequence_Name -eq [String]::Empty))
  {Throw "Nom de squence invalide. La valeur de Sequence_Name doit tre renseigne."}

 write-Debug "Test isTypeEqual termin."
 
#Les paramtres de type contraint sont initialiss  une valeur par dfaut, par exemple pour les entiers  zro, 
#on ne peut donc pas les tester sur la valeur $null afin de dterminer si un paramtre est prsent ou non sur la ligne de commande. 
#Et dans certains cas la valeur attribue par dfaut peut ne pas tre souhaitable. 
#L'objet $MyInvocation.CommandLineParameters peut nous aider  tester ces cas de figure.        
 if (($MyInvocation.CommandLineParameters.Increment_By -ne $null) -and ($Increment_By -eq 0) ) 
  {Throw "La valeur de Increment_By doit tre un entier diffrent de zro."}
     
# Valeur par dfaut
# Si on ne spcifie aucun paramtre alors on cre une squence ascendante qui dbute  1
# est incrmente de 1 sans limite de valeur suprieure (autre que celle du type utilis)   
#Si on spcifie seulement -Increment_By -1 on cre une squence descendante qui dbute  -1
# est dcrmente de -1 sans limite de valeur infrieure (autre que celle du type utilis)
 write-Debug ""
 write-Debug "Les valeurs par dfaut pour la squence de type sont :"

#Ici si $Increment_By n'est pas renseign il vaut par dfaut 1, on ne peut donc avoir 0 comme valeur renvoye par la mthode Sign 
$local:Signe=[System.Math]::Sign($Increment_By)      
 write-Debug "`tIncrement_By $Increment_By"
 write-Debug "`tSigne $Signe (1= positif  -1= ngatif)"
 
 if ( ($Signe -eq 1) -and
      ($MyInvocation.CommandLineParameters.Start_With -eq $null) -and
      ($MyInvocation.CommandLineParameters.MaxValue -eq $null) -and 
      ($MyInvocation.CommandLineParameters.MinValue -eq $null)
     )
      { write-Debug "`tSquence ascendante. Valeur par dfaut."
        $Start_With=1     
        $MaxValue=[int]::MaxValue
        $MinValue=1
      }
 elseif 
    ( ($Signe -eq -1) -and
      ($MyInvocation.CommandLineParameters.Start_With -eq $null) -and
      ($MyInvocation.CommandLineParameters.MaxValue -eq $null) -and
      ($MyInvocation.CommandLineParameters.MinValue -eq $null)
     )
      { write-Debug "`tSquence descendante. Valeur par dfaut."
        $Start_With=-1     
        $MaxValue=-1
        $MinValue=[int]::MinValue
      }
 else
  { 
    write-Debug "`t* Pas de valeur par dfaut. *"
 
    # Si MaxValue n'est pas spcifi on indique la valeur maximum pour une squence ascendante 
    # sinon 1 pour une squence descendante    
   if ($MyInvocation.CommandLineParameters.MaxValue -eq $null)
    { $MaxValue=[int]::MaxValue 
      if ($local:Signe -eq -1)
       {$MaxValue=-1 }
    }
    # Si MinValue n'est pas spcifi on indique la valeur 1 pour une squence ascendante 
    # sinon la valeur minimum  pour une squence descendante    
   if ($MyInvocation.CommandLineParameters.MinValue -eq $null)
    { $MinValue=1
      if ($Signe -eq -1)
       {$MinValue=[int]::MinValue }
    }
     
     # Si Start_With n'est pas spcifi on indique, pour une squence ascendante, la valeur minimum de la squence 
     # ou pour une squence descendante la valeur maximum de la squence.
   if ($MyInvocation.CommandLineParameters.Start_With -eq $null) 
    {
     switch ($local:Signe)
      { 
        1  {$Start_With=$MinValue}
       -1  {$Start_With=$MaxValue}
      }#switch    
    }#If
 }#else 
 
 write-Debug "Start_With $Start_With"
 write-Debug "MaxValue $MaxValue"
 write-Debug "MinValue $MinValue"

 if ($MyInvocation.CommandLineParameters.Increment_By -ne $null)
  { 
   if ($Cycle.Ispresent) 
    { #Dans ce cas selon le signe du paramtre $Increment_By on doit prciser soit Minvalue soit MaxValue  
     switch ($local:Signe)
      { 
       1  { if ($MyInvocation.CommandLineParameters.MinValue -eq $null) 
             { Throw "Squence ascendante cyclique pour laquelle vous devez spcifier MinValue."}
          }
      -1  { if ($MyInvocation.CommandLineParameters.MaxValue -eq $null)
            { Throw "Squence descendante cyclique pour laquelle vous devez spcifier MaxValue."}
          }
      default  { Throw "Analyse errone!"}
      }#switch
    }#if cycle
  }#if increment
 write-Debug "Test de la variable Cycle termin."      
 write-Debug "[Test de validit de la squence]"     

 # MINVALUE must be less than or equal to START WITH and must be less than MAXVALUE. 
 if (!($MinValue -le $Start_With ))
  { Throw "Start_With($Start_With) ne peut pas tre infrieur  MinValue($MinValue)."}
 elseif (!($MinValue -lt $MaxValue))
  { Throw "MinValue($MinValue) doit tre infrieure  MaxValue($MaxValue)."}
 # MAXVALUE must be equal to or greater than START WITH and must be greater than MINVALUE.
 if (!($MaxValue -ge $Start_With))
   { Throw "Start_With($Start_With) ne peut pas tre suprieur  MaxValue($MaxValue)."}
 elseif (!($MaxValue -gt $MinValue))
  { Throw "MaxValue($MaxValue) doit tre suprieure  MinValue($MinValue)."}
  
  #On test s'il est possible d'itrer au moins une fois.
  #La construction suivant est valide mais pour un seul chiffre :
  # $Sq=Create-Sequence $N $C -inc 1 -min 0 -max 5 -start 5
  # $Sq=Create-Sequence $N $C -inc -1 -min 0 -max 2 -start 0
    
  #Start_with n'est pas pris en compte dans ce calcul, on autorise donc un squence proposant un seul nombre.
  #Si dans ce cas on prcise le switch -Cycle on a bien plusieurs itrations :
  # $Sq=Create-Sequence $N $C -inc 1 -min 0 -max 5 -start 5 -cycle

  #On cast $Increment_By car sa valeur peut tre gale  [int]::MinValue, d'o une exception lors de l'appel  Abs
  # La documentation Oracle prcise l'oprateur -lt mais dans ce cas la squence suivante est impossible :
  #  Create-Sequence $N $C -min 1 -max 2 
 if (!([system.Math]::Abs([Long]$Increment_By) -le ($MaxValue-$MinValue)) )
  { Throw "Increment_By($Increment_By) doit tre infrieur ou gale  MaxValue($MaxValue) moins MinValue($MinValue)."} 

#Cration de la squence
 $Sequence= new-object System.Management.Automation.PsObject
 # Ajout des proprits en R/O, le code est cr dynamiquement
$MakeReadOnlyMember=@"
`$Sequence | add-member ScriptProperty Name         -value {"$Sequence_Name"}   -Passthru|`
             add-member ScriptProperty Comment      -value {"$Comment"}         -Passthru|`
             add-member ScriptProperty Increment_By -value {[int]$Increment_By} -Passthru|`
             add-member ScriptProperty MaxValue     -value {[int]$MaxValue}     -Passthru|`
             add-member ScriptProperty MinValue     -value {[int]$MinValue}     -Passthru|`
             add-member ScriptProperty Start_With   -value {[int]$Start_With}   -Passthru|`
             add-member ScriptProperty Cycle        -value {`$$Cycle}           -Passthru|`
             add-member ScriptProperty CurrVal      -value {[int]$Start_With}
"@
  Invoke-Expression $MakeReadOnlyMember
  
  #La mthode NextVal renvoie en fin de traitement la valeur courante du membre Currval
 $Sequence | add-member ScriptMethod NextVal {   
                          $NewValue=$this.CurrVal + $this.Increment_By
                          write-debug "this $this"
                          write-debug "NewValue $NewValue"
                           
                           #Dclaration paramtre pour la redfinition du membre CurrVal
                           #Le paramtre -Force annule et remplace la dfinition du membre spcifi
                          $RazMember="`$this | add-member -Force ScriptProperty CurrVal  -value {0}"
                                                                                                  
                          switch ([System.Math]::Sign($this.Increment_By))
                          {  
                              #MAXVALUE cannot be made to be less than the current value
                             1  { if ($NewValue -gt  $this.MaxValue)
                                  { write-debug "Borne maximum atteinte."
                                    
                                    if ($this.Cycle -eq $true) #Dans ce cas on recommence
                                     { #On construit (par formatage) la dfinition du membre CurrVal 
                                       #puis on reconstruit le membre.
                                      Invoke-Expression ($RazMember -F "{[int]$this.MinValue}")
                                     } 
                                    else {Throw "La squence $($this.Name).Nextval a atteint la valeur maximum autorise."}
                                  }
                                  else { Invoke-Expression ($RazMember -F "{[int]$NewValue}")}
                                }#Squence ascendante

                              #MINVALUE cannot be made to exceed the current value 
                            -1  { if ($NewValue -lt  $this.MinValue)
                                  { write-debug "Borne minimum atteinte."
                                    if ($this.Cycle -eq $true)
                                     {Invoke-Expression ($RazMember -F "{[int]$this.MaxValue}") }
                                    else {Throw "La squence $($this.Name).Nextval a atteint la valeur minimum autorise."}
                                  }
                                  else {Invoke-Expression ($RazMember -F "{[int]$NewValue}") }
                                }#Squence descendante
                             else {Throw "Erreur dans la mthode Nextval."}    
                          }#switch
                           #Renvoi la nouvelle valeur
                          $this.CurrVal
                          write-debug "this $this"
                        }

 write-Debug "[Valeurs de la squence]"
 write-Debug "Sequence_Name $Sequence_Name"
 write-Debug "Comment $Comment"
 write-Debug "Increment_By $Increment_By" 
 write-Debug "MaxValue $MaxValue"
 write-Debug "MinValue $MinValue"
 write-Debug "Start_With $Start_With"  
 write-Debug "Cycle $Cycle"        
 write-Debug "Current $Current"
 write-Debug "----"
 write-Debug  $Sequence

  #Spcifie l'affichage des proprits par dfaut
  #On vite ainsi l'usage d'un fichier de type .ps1xml
  #From http://poshoholic.com/2008/07/05/essential-powershell-define-default-properties-for-custom-objects/
 $DefaultProperties =@(
  'Name',
  'CurrVal', 
  'Increment_By',
  'MaxValue',
  'MinValue',
  'Start_With',
  'Cycle',
  'Comment')
 $DefaultPropertySet=New System.Management.Automation.PSPropertySet('DefaultDisplayPropertySet',[string[]]$DefaultProperties)
 $PSStandardMembers=[System.Management.Automation.PSMemberInfo[]]@($DefaultPropertySet)
 $Sequence|Add-Member MemberSet PSStandardMembers $PSStandardMembers

 return $Sequence
}

# SIG # Begin signature block
# MIIFQgYJKoZIhvcNAQcCoIIFMzCCBS8CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUhevflw0Rn4etFAsnBtI81oB4
# cGygggL8MIIC+DCCAmWgAwIBAgIQGpdOSYj2EbRAxGqmgFirkTAJBgUrDgMCHQUA
# MHsxeTB3BgNVBAMecABMAGEAdQByAGUAbgB0ACAARABhAHIAZABlAG4AbgBlACAA
# YQB1AHQAbwByAGkAdADpACAAZABlACAAYwBlAHIAdABpAGYAaQBjAGEAdABpAG8A
# bgAgAHIAYQBjAGkAbgBlACAAbABvAGMAYQBsAGUwHhcNMDcwNzAxMTMyNjU5WhcN
# MzkxMjMxMjM1OTU5WjA2MTQwMgYDVQQDEytMYXVyZW50IERhcmRlbm5lIGNlcnRp
# ZmljYXQgcG91ciBQb3dlclNoZWxsMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKB
# gQCfu+w77PdPXH8+C41SaR48k/DPql1EPDL8O4gArRwbKH/McvXmXCKEwaWYlFi7
# w8F4CwO/kBswENa+0X3OXLUOK0LyQmJP7VQqA+mT4Up+a5Z3mcsRd1Out+OzfOuR
# hium1zhE1/MlIqEK6hnMl/A/bkc4SCFfdiJeZc83tNkGXwIDAQABo4HJMIHGMBMG
# A1UdJQQMMAoGCCsGAQUFBwMDMIGuBgNVHQEEgaYwgaOAEB0w3fWBsN0e2nTMjcGF
# f8qhfTB7MXkwdwYDVQQDHnAATABhAHUAcgBlAG4AdAAgAEQAYQByAGQAZQBuAG4A
# ZQAgAGEAdQB0AG8AcgBpAHQA6QAgAGQAZQAgAGMAZQByAHQAaQBmAGkAYwBhAHQA
# aQBvAG4AIAByAGEAYwBpAG4AZQAgAGwAbwBjAGEAbABlghAb5Gp5W/j2oEZ4E3Mn
# LmpOMAkGBSsOAwIdBQADgYEAoEjrLmRoEscvqLGp6RXFH55NjCul7e118oWxlpHt
# hcme2FZVN0vNB0Xqa+A3YU4QyYhYNeBaJ/gsgv1MC7PnuBR2ek58mTwVa6WlVNrn
# KK8A7P3MRVCOGVYkOiw5xttWFvPPph1YG1CAAwAwSI+nIfJCxyJDceOvvbCoV+US
# FLgxggGwMIIBrAIBATCBjzB7MXkwdwYDVQQDHnAATABhAHUAcgBlAG4AdAAgAEQA
# YQByAGQAZQBuAG4AZQAgAGEAdQB0AG8AcgBpAHQA6QAgAGQAZQAgAGMAZQByAHQA
# aQBmAGkAYwBhAHQAaQBvAG4AIAByAGEAYwBpAG4AZQAgAGwAbwBjAGEAbABlAhAa
# l05JiPYRtEDEaqaAWKuRMAkGBSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKAC
# gAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsx
# DjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBQZ20Yp3aplMijqRGksWuy8
# m5nwkzANBgkqhkiG9w0BAQEFAASBgJP7am7iRbjncSa70RQRqNqyRhmrrRAXBe5J
# DBkrXVV04c0rzjDjJ83WNSq/IpwRPxbFQbF8Qz3orJWvx6d1PhQgBkrSS/F+vD77
# TMMJOe1h6vl6jwHwURqOpwXdKSikBOvC9AT0DCowdMoGmLbkZLW+YMOVmv3OAfum
# mCwTMZ4P
# SIG # End signature block
